/* This file was generated by syncqt. */
#ifndef QT_QTQUICKCONTROLS2MATERIAL_VERSION_H
#define QT_QTQUICKCONTROLS2MATERIAL_VERSION_H

#define QTQUICKCONTROLS2MATERIAL_VERSION_STR "6.11.0"

#define QTQUICKCONTROLS2MATERIAL_VERSION 0x0601100

#endif // QT_QTQUICKCONTROLS2MATERIAL_VERSION_H
