#ifndef QT_FEATURES_QuickTemplates2_src_quicktemplates_qtquicktemplates2_config_h_H
#define QT_FEATURES_QuickTemplates2_src_quicktemplates_qtquicktemplates2_config_h_H


#endif // QT_FEATURES_QuickTemplates2_src_quicktemplates_qtquicktemplates2_config_h_H
